import datetime
from lcwc.incident import Incident
from lcwc.category import IncidentCategory
from collections import namedtuple

from lcwc.unit import Unit

Coordinates = namedtuple("Coordinates", ["longitude", "latitude"])


class ArcGISIncident(Incident):
    """Represents an incident from the live ArcGIS REST API"""

    def __init__(
        self,
        category: IncidentCategory,
        date: datetime,
        description: str,
        municipality: str,
        intersection: str,
        number: int,
        priority: int,
        agency: str,
        public: bool,
        coordinates: Coordinates,
        units: list[Unit] = [],
    ) -> None:
        super().__init__(category, date, description, municipality, intersection, units)
        self._incident_number = number
        self._priority = priority
        self._agency = agency
        self._public = public
        self._coordinates = coordinates

    @property
    def number(self) -> int:
        """Returns the number of the incident"""
        return self._incident_number

    @property
    def priority(self) -> int:
        """Returns the priority of the incident"""
        return self._priority

    @property
    def agency(self) -> str:
        """Returns the agency of the incident"""
        return self._agency

    @property
    def coordinates(self) -> Coordinates:
        """Returns the coordinates of the incident"""
        return self._coordinates

    @property
    def public(self) -> bool:
        """Returns whether the incident is public"""
        return self._public



from abc import ABC
import datetime

from lcwc.category import IncidentCategory
from lcwc.unit import Unit


class Incident(ABC):
    """Represents an incident"""

    def __init__(
        self,
        category: IncidentCategory,
        date: datetime,
        description: str,
        municipality: str,
        intersection: str,
        units: list[Unit] = [],
    ) -> None:
        self._category = category
        self._date = date
        self._description = description
        self._municipality = municipality
        self._intersection = intersection
        self._units = units

    @property
    def category(self) -> IncidentCategory:
        """Returns the category of the incident"""
        return self._category

    @property
    def date(self) -> datetime:
        """Returns the date and time of the incident"""
        return self._date

    @property
    def description(self) -> str:
        """Returns the incident description"""
        return self._description

    @property
    def municipality(self) -> str:
        """Returns the municipality of the incident location"""
        return self._municipality

    @property
    def intersection(self) -> str:
        """Returns the intersection of the incident location"""
        return self._intersection

    @property
    def units(self) -> list[Unit]:
        """Returns a list of units responding to the incident"""
        return self._units



# custom class to handle the guid attribute
# a bit usless on its own but might be useful for statically identifying incidents
# since apparently every detail is subject to change except maybe the timestamp
import datetime
from lcwc.category import IncidentCategory
from lcwc.incident import Incident
from lcwc.unit import Unit


class FeedIncident(Incident):
    """Represents an incident from the live incident feed"""

    def __init__(
        self,
        category: IncidentCategory,
        date: datetime,
        description: str,
        municipality: str,
        intersection: str,
        units: list[Unit] = [],
        guid: str = None,
    ) -> None:
        super().__init__(category, date, description, municipality, intersection, units)
        self._guid = guid

    @property
    def guid(self) -> str:
        """Returns the guid of the incident"""
        return self._guid



class Unit:
    def __init__(self, name: str):
        """Constructor.

        :param str name: The name of the unit
        """

        self._name = name

    @property
    def name(self) -> str:
        """Returns the name of the unit"""
        return self._name

    def __str__(self) -> str:
        return self._name

    def __eq__(self, __value: object) -> bool:
        if not isinstance(__value, Unit):
            return NotImplemented
        return self._name == __value.name
